#!/usr/bin/env ruby

require 'net/imap'
require 'net/smtp'
require 'cgi'
require 'kconv'

module Net
  class IMAP
    def fetch_attr(seq,s)
      fetch(seq,s)[0].attr[s]
    end
  end
end

class WebMail
  @@cgiurl = File.basename($0)

  def initialize(server,user,password)
    @server = server
    @user = user
    @password = password

    @cgi = CGI.new("html3")
    @cmd = @cgi['cmd'].to_s
    @mailbox = @cgi['mailbox'].to_s
    @mailbox = "INBOX" if @mailbox == ''
    @start = @cgi['start'].to_s.to_i
    @message_id = @cgi['message_id'].to_s
    @seq = @cgi['seq'].to_s.to_i
  end

  def init_imap
    @imap = Net::IMAP.new(@server,143)
    @imap.authenticate('CRAM-MD5',@user,@password)
    # @imap.login(userConfig, passConfig)
    @imap.examine(@mailbox)
  end

  def list
    init_imap
    messages = @imap.status(@mailbox, ["MESSAGES"])["MESSAGES"]
    (1..messages).to_a.reverse[@start..@start+9].collect { |seq| 
      subject = @imap.fetch_attr(seq,"BODY[HEADER.FIELDS (SUBJECT)]").gsub(/Subject:\s+(.*)/,'\1')
      subject = "(no title)" if subject == ''
      from = @imap.fetch_attr(seq,"BODY[HEADER.FIELDS (FROM)]").gsub(/From:\s+(.*)/,'\1').gsub(/.*<(.*)>.*/,'\1')
      message_id= @imap.fetch_attr(seq,"ENVELOPE").message_id
      read_cgi = "#{@@cgiurl}?mailbox=#{@mailbox}&message_id=#{message_id}&cmd=read"
      @cgi.a('href' => read_cgi){ subject } + "(#{from})" + @cgi.br
    }.join +
      (@start >= 10 ? 
       "<a href=\"#{@@cgiurl}?mailbox=#{@mailbox}&start=#{@start-10}\"> </a> " : "") +
      "<a href=\"#{@@cgiurl}?mailbox=#{@mailbox}&start=#{@start+10}\"></a>"
  end

  def read
    init_imap
    seq = @imap.search(["HEADER","MESSAGE-ID",@message_id])[0]
    from = @imap.fetch_attr(seq,"BODY[HEADER.FIELDS (FROM)]").gsub(/</,"&lt;")
    date = @imap.fetch_attr(seq,"BODY[HEADER.FIELDS (DATE)]")
    write_cgi = "#{@@cgiurl}?mailbox=#{@mailbox}&seq=#{seq}&cmd=write"
    date + @cgi.br + @cgi.a('href' => write_cgi){ from } + @cgi.br + @cgi.hr +
      @imap.fetch_attr(seq, "BODY[1.TEXT]").collect {|line| line + @cgi.br}.join
  end

  def write
    init_imap
    to = @imap.fetch_attr(@seq,"BODY[HEADER.FIELDS (FROM)]").gsub(/From:\s+(.*)/,'\1')
    from = "#{@user}@#{@server}"
    subject = "Re: " + @imap.fetch_attr(@seq,"BODY[HEADER.FIELDS (SUBJECT)]").gsub(/Subject:\s+(.*)/,'\1')

    text = (@seq == 0 ? '' :
	    @imap.fetch_attr(@seq,"BODY[1.TEXT]").collect {|line| "> " + line}.join)
    @cgi.form('method' => 'post', 'action' => "#{@@cgiurl}"){
      @cgi.input('type' => 'hidden', 'name' => 'cmd', 'value' => 'deliver') +
	'To: ' + @cgi.input('type' => 'text', 'size' => '40', 'name' => 'to', 'value' => to) + @cgi.br +
	'From: ' + @cgi.input('type' => 'text', 'size' => '40', 'name' => 'from', 'value' => from) + @cgi.br +
	'Subject: ' + @cgi.input('type' => 'text', 'size' => '40', 'name' => 'subject', 'value' => subject) + @cgi.br +
	@cgi.textarea('name' => 'text', 'rows' => '20', 'cols' => '60', 'wrap' => 'off'){ text } + @cgi.br +
	@cgi.submit('value' => 'send')
    }
  end

  def mime_b (s)
    s.tojis.gsub(/\033\$B([^\033]*)\033\(B/){ |m|
      '=?iso-2022-jp?B?' + [m].pack('m').chomp + '?='
    }
  end

  def deliver
    from = mime_b(@cgi['from'].to_s)
    to = mime_b(@cgi['to'].to_s)
    Net::SMTP.start(@server, 25) {|smtp|
      body =  "To: #{to}\n"
      body << "From: #{from}\n"
      body << "Subject: #{mime_b(@cgi['subject'].to_s)}\n"
      body << "X-Mailer: Compact WebMail by masui@example.com\n\n"
      body << "#{@cgi['text'].to_s.tojis}"
      smtp.send_mail(body,from,to)
    }
    @cgi.meta('HTTP-EQUIV' => 'refresh', 'CONTENT' => "1;URL=#{@@cgiurl}")
  end
  
  def do_command
    cmd = @cmd.intern # write, read ΥޥȽ
    body = (self.respond_to?(cmd) ? self.send(cmd) : list)
    @cgi.out {
      @cgi.html {
	@cgi.head { @cgi.title { "WebMail (#{@cmd})" }} +
	  @cgi.body { body.tosjis }
      }
    }
  end
end

webmail = WebMail.new('example.com','masui','mypasswd')
webmail.do_command
